package com.dzeio.openhealth.extensions

import androidx.activity.result.ActivityResultCallback
import androidx.activity.result.contract.ActivityResultContract
import androidx.fragment.app.Fragment
import com.dzeio.openhealth.data.weight.Weight
import kotlinx.coroutines.flow.Flow

/**
 * Extension Schema
 *
 * Version: 0.2.0
 */
interface Extension : ActivityResultCallback<Any> {

    data class TaskProgress<T>(
        /**
         * value indicating the current status of the task
         */
        val state: TaskState = TaskState.INITIALIZATING,

        /**
         * value between 0 and 100 indicating the progress for the task
         */
        val progress: Float? = null,

        /**
         * Additionnal message that will be displayed when the task has ended in a [TaskState.CANCELLED] or [TaskState.ERROR] state
         */
        val statusMessage: String? = null,

        /**
         * Additional information
         */
        val additionalData: T? = null
    )

    enum class TaskState {
        /**
         * define the task as being preped
         */
        INITIALIZATING,

        /**
         * Define the task a bein worked on
         */
        WORK_IN_PROGRESS,

        /**
         * define the task as being done
         */
        DONE,

        /**
         * Define the task as being cancelled
         */
        CANCELLED,

        /**
         * define the task as being ended with an error
         */
        ERROR
    }

    enum class Data {
        /**
         * Special case to handle basic errors from other activities
         */
        NOTHING,
        WEIGHT,
        STEPS

        /**
         * Google Fit:
         *
         * STEP_COUNT_CUMULATIVE
         * ACTIVITY_SEGMENT
         * SLEEP_SEGMENT
         * CALORIES_EXPENDED
         * BASAL_METABOLIC_RATE
         * POWER_SAMPLE
         * HEART_RATE_BPM
         * LOCATION_SAMPLE
         */
    }

    /**
     * the permissions necessary for the extension to works
     */
    val permissions: Array<String>

    /**
     * the Source ID
     *
     * DO NOT CHANGE IT AFTER THE EXTENSION IS IN PRODUCTION
     */
    val id: String

    /**
     * The Extension Display Name
     */
    val name: String

    /**
     * the different types of data handled by the extension
     */
    val data: Array<Data>

    /**
     * Enable the extension, no code is gonna be run before
     */
    fun enable(fragment: Fragment): Boolean

    /**
     * return if the extension is already connected to remote of not
     */
    suspend fun isConnected(): Boolean

    /**
     * Return if the extension is runnable on the device
     */
    fun isAvailable(): Boolean

    /**
     * try to connect to remote
     */
    suspend fun connect(): Boolean
    
    val contract: ActivityResultContract<*, *>?
    val requestInput: Any?
    suspend fun importWeight(): Flow<TaskProgress<ArrayList<Weight>>>

    /**
     * function run when outgoing sync is enabled and new value is added
     * or manual export is launched
     */
    suspend fun exportWeights(weight: Array<Weight>): Flow<TaskProgress<Unit>>

//    fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) = Unit


    suspend fun permissionsGranted(): Boolean
}
