package com.dzeio.openhealth.extensions

import android.Manifest
import android.util.Log
import androidx.activity.result.contract.ActivityResultContract
import androidx.activity.result.contract.ActivityResultContracts
import androidx.fragment.app.Fragment
import com.dzeio.openhealth.core.Observable
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.utils.PermissionsManager
import com.google.android.gms.auth.api.signin.GoogleSignIn
import com.google.android.gms.fitness.Fitness
import com.google.android.gms.fitness.FitnessOptions
import com.google.android.gms.fitness.data.DataType
import com.google.android.gms.fitness.request.DataReadRequest
import kotlinx.coroutines.ExperimentalCoroutinesApi
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.channelFlow
import kotlinx.coroutines.runBlocking
import kotlinx.coroutines.suspendCancellableCoroutine
import java.util.Calendar
import java.util.Date
import java.util.TimeZone
import java.util.concurrent.TimeUnit

class GoogleFitExtension : Extension {
    companion object {
        const val TAG = "GoogleFitConnector"
    }

    override val id = "GoogleFit"
    override val name = "Google Fit"

    override val permissions = arrayOf(
        Manifest.permission.ACCESS_FINE_LOCATION
    )

    override val data: Array<Extension.Data> = arrayOf(
        Extension.Data.WEIGHT
    )

    override suspend fun isConnected(): Boolean =
        GoogleSignIn.hasPermissions(getGoogleAccount(), fitnessOptions)


    private val fitnessOptions = FitnessOptions.builder()
        .addDataType(DataType.TYPE_WEIGHT)
//        .addDataType(DataType.TYPE_STEP_COUNT_CUMULATIVE)
//        .addDataType(DataType.TYPE_CALORIES_EXPENDED)
        .build()

    private val connectionStatus = Observable(false)

    private lateinit var fragment: Fragment

    override fun isAvailable(): Boolean = true

    override fun enable(fragment: Fragment): Boolean {
        this.fragment = fragment
        return true
    }

    @OptIn(ExperimentalCoroutinesApi::class)
    override suspend fun connect(): Boolean {

        if (isConnected()) {
            return true
        }

        return suspendCancellableCoroutine { cancellableContinuation ->
            Log.d(this.name, "Signing In")
            GoogleSignIn.requestPermissions(
                fragment,
                124887,
                getGoogleAccount(), fitnessOptions
            )
            connectionStatus.addOneTimeObserver { it: Boolean ->
                cancellableContinuation.resume(it) {

                }
            }
        }
    }

    override val contract: ActivityResultContract<*, Map<String, @JvmSuppressWildcards Boolean>>? = ActivityResultContracts.RequestMultiplePermissions()
    override val requestInput = permissions

    private fun getGoogleAccount() = GoogleSignIn.getAccountForExtension(fragment.requireContext(), fitnessOptions)

    private val timeRange by lazy {
        val calendar = Calendar.getInstance(TimeZone.getTimeZone("UTC"))
        calendar.time = Date()
        val endTime = calendar.timeInMillis

        // Set year to 2013 to be sure to get data from when Google Fit Started to today
        calendar.set(Calendar.YEAR, 2013)
        val startTime = calendar.timeInMillis
        return@lazy arrayOf(startTime, endTime)
    }

    override suspend fun importWeight(): Flow<Extension.TaskProgress<ArrayList<Weight>>> =
        channelFlow {
            send(
                Extension.TaskProgress(
                    Extension.TaskState.INITIALIZATING
                )
            )

            val type = DataType.TYPE_WEIGHT
            val timeUnit = TimeUnit.MILLISECONDS

            val request = DataReadRequest.Builder()
                .read(type)
                .setTimeRange(timeRange[0], timeRange[1], timeUnit)
                .build()

            Fitness.getHistoryClient(
                fragment.requireContext(),
                GoogleSignIn.getAccountForExtension(fragment.requireContext(), fitnessOptions)
            )
                .readData(request)
                .addOnSuccessListener { response ->
                    val weights: ArrayList<Weight> = ArrayList()
                    var index = 0
                    var total = response.dataSets.size
                    for (dataset in response.dataSets) {
                        total += dataset.dataPoints.size - 1
                        for (dataPoint in dataset.dataPoints) {
                            total += dataPoint.dataType.fields.size - 1
                            for (field in dataPoint.dataType.fields) {
                                val weight = Weight().apply {
                                    timestamp = dataPoint.getStartTime(TimeUnit.MILLISECONDS)
                                    weight = dataPoint.getValue(field).asFloat()
                                    source = this@GoogleFitExtension.id
                                }
                                weights.add(weight)
                                runBlocking {
                                    send(
                                        Extension.TaskProgress(
                                            Extension.TaskState.WORK_IN_PROGRESS,
                                            progress = index++ / total.toFloat()
                                        )
                                    )
                                }
                            }
                        }
                    }
                    runBlocking {
                        send(
                            Extension.TaskProgress(
                                Extension.TaskState.DONE,
                                additionalData = weights
                            )
                        )
                    }
                }
                .addOnFailureListener {
                    runBlocking {
                        send(
                            Extension.TaskProgress(
                                Extension.TaskState.ERROR,
                                statusMessage = it.localizedMessage ?: it.message ?: "Unknown error"
                            )
                        )
                    }
                }


        }

    override suspend fun exportWeights(weight: Array<Weight>): Flow<Extension.TaskProgress<Unit>> {
        TODO("Not yet implemented")
    }

    override suspend fun permissionsGranted(): Boolean {
        return PermissionsManager.hasPermission(this.fragment.requireContext(), permissions)
    }

    override fun onActivityResult(result: Any) {
        if ((result as Map<*, *>).containsValue(false)) {
            return
        }

        connectionStatus.value = true
    }

}
