package com.dzeio.openhealth.extensions

import android.Manifest
import android.os.Build
import androidx.activity.result.contract.ActivityResultContract
import androidx.annotation.RequiresApi
import androidx.fragment.app.Fragment
import androidx.health.connect.client.HealthConnectClient
import androidx.health.connect.client.PermissionController
import androidx.health.connect.client.permission.HealthPermission
import androidx.health.connect.client.records.HeartRateRecord
import androidx.health.connect.client.records.StepsRecord
import androidx.health.connect.client.records.WeightRecord
import androidx.health.connect.client.request.ReadRecordsRequest
import androidx.health.connect.client.time.TimeRangeFilter
import com.dzeio.openhealth.core.Observable
import com.dzeio.openhealth.data.weight.Weight
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.channelFlow
import kotlinx.coroutines.runBlocking
import java.time.Instant

@RequiresApi(Build.VERSION_CODES.P)
class HealthConnectExtension : Extension {
    companion object {
        const val TAG = "HealthConnectExtension"
    }

    // build a set of permissions for required data types
    val PERMISSIONS =
        setOf(
            HealthPermission.createReadPermission(HeartRateRecord::class),
            HealthPermission.createWritePermission(HeartRateRecord::class),
            HealthPermission.createReadPermission(StepsRecord::class),
            HealthPermission.createWritePermission(StepsRecord::class)
        )


    override val id = "HealthConnect"
    override val name = "Health Connect"

    override val permissions = arrayOf(
        Manifest.permission.ACCESS_FINE_LOCATION
    )

    override val requestInput = PERMISSIONS

    override val data: Array<Extension.Data> = arrayOf(
        Extension.Data.WEIGHT
    )

    override suspend fun isConnected(): Boolean = true



    private val connectionStatus = Observable(false)

    private lateinit var fragment: Fragment
    private lateinit var client: HealthConnectClient

    override fun isAvailable(): Boolean {
        return HealthConnectClient.isAvailable(fragment.requireContext())
    }

    override fun enable(fragment: Fragment): Boolean {
        this.fragment = fragment
        if (!isAvailable()) {
            return false
        }

        this.client = HealthConnectClient.getOrCreate(fragment.requireContext())

        return true
    }

    override suspend fun connect(): Boolean = true

    override suspend fun importWeight(): Flow<Extension.TaskProgress<ArrayList<Weight>>> =
        channelFlow {
            send(
                Extension.TaskProgress(
                    Extension.TaskState.INITIALIZATING
                )
            )

            val response = client.readRecords(
                ReadRecordsRequest(
                    WeightRecord::class,
                    timeRangeFilter = TimeRangeFilter.before(Instant.now())
                )
            )

            val weights: ArrayList<Weight> = ArrayList()
            var index = 0
            for (record in response.records) {
                val weight = Weight().apply {
                    timestamp = record.time.toEpochMilli()
                    weight = record.weight.inKilograms.toFloat()
                    source = this@HealthConnectExtension.id
                }
                weights.add(weight)
                runBlocking {
                    send(
                        Extension.TaskProgress(
                            Extension.TaskState.WORK_IN_PROGRESS,
                            progress = index++ / response.records.size.toFloat()
                        )
                    )
                }
            }
            runBlocking {
                send(
                    Extension.TaskProgress(
                        Extension.TaskState.DONE,
                        additionalData = weights
                    )
                )
            }

        }

    override suspend fun exportWeights(weight: Array<Weight>): Flow<Extension.TaskProgress<Unit>> {
        TODO("Not yet implemented")
    }

    override fun onActivityResult(result: Any) {
        if ((result as Set<*>).containsAll(this.PERMISSIONS)) connectionStatus.value = true
        // signIn(Data.values()[requestCode])
    }

    override val contract: ActivityResultContract<Set<HealthPermission>, Set<HealthPermission>>
        get() = PermissionController.createRequestPermissionResultContract()

    override suspend fun permissionsGranted(): Boolean {
        return this.client.permissionController.getGrantedPermissions(this.PERMISSIONS).containsAll(this.PERMISSIONS)
    }
}
